@@ -59,7 +59,7 @@ def test_best_response_plurality():
         pivot_ki_easy_or_tight = True
         pivot_jk_easy_or_tight = False
         pivot_kj_easy_or_tight = False
-        threshold_utility = 1
+        utility_threshold = 1
         justification = Plurality analysis
         ballot = a
     """
@@ -75,7 +75,7 @@ def test_best_response_is_j():
 def test_best_response_is_utility_dependent():
     tau = TauVector({'a': 1 / 5, 'b': 3 / 5, 'c': 1 / 5}, voting_rule=PLURALITY)
     best_response = BestResponsePlurality(tau, 'abc')
-    assert best_response.threshold_utility == Fraction(1, 2)
+    assert best_response.utility_threshold == Fraction(1, 2)
     assert best_response.ballot == UTILITY_DEPENDENT
 
 
